@@ -7,6 +7,7 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
         super().__init__(model)
         self._convert_to_min_problem()
         self._defining_points = dict()
+        self._decision_variable_map = dict()
         self._init_M()
         self._init_ideal_point()
 
@@ -22,6 +23,10 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
     def weakly_dominates(self, left, right):
         return all((x <= y for x, y in zip(left, right)))
 
+    def dominates(self, left, right):
+        if left == right: return False
+        return self.weakly_dominates(left, right)
+
     # Algorithm 1
     def _update_search_region(self, new_point, search_region):
         # Output
@@ -100,14 +105,22 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
             target.addLConstr(new_expression, constr.Sense, constr.RHS, name=constr.ConstrName)
         target.update()
 
-    def _kth_obj_model(self, k):
-        new_model = gp.Model(f"objective-{k}")
+    def _set_start_values(self, model, values):
+        for varname in values:
+            model.getVarByName(varname).setAttr(GRB.Attr.Start, values[varname])
+
+    def _new_empty_objective_model(self):
+        new_model = gp.Model()
         new_model.Params.OutputFlag = 0  # Suppress console output
         self._copy_vars_to(self._model, new_model)
-        self._copy_objective_to(self._model, new_model, k, 0)
         self._copy_constraints_to(self._model, new_model)
         return new_model
 
+    def _kth_obj_model(self, k):
+        new_model = self._new_empty_objective_model()
+        self._copy_objective_to(self._model, new_model, k, 0)
+        return new_model
+
     def _init_ideal_point(self):
         point = []
         for k in range(self._model.NumObj):
@@ -147,60 +160,57 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
     def _eval_objective_given_model(self, model, objective):
         return self._eval_linexpr_for_values(objective, self._var_values_by_name_dict(model))
 
-    def _construct_subproblem(self, k, u):
-        # For the Two-Stage Approach, solve the first sub problem and return the second stage.
-        # For the Direct Approach, return the weighted single objective problem.
-        # First, we implement the Two-Stage Approach.
+    def _find_and_set_start_values(self, model, k, u):
+        if (k,u) in self._defining_points:
+            N_ku = self._defining_points[(k,u)]
+            if len(N_ku) > 0:
+                feasible_nd = list(N_ku)[0]
+                if feasible_nd in self._decision_variable_map:
+                    feasible_variables = self._decision_variable_map[feasible_nd]
+                    self._set_start_values(model, feasible_variables)
 
-        # First stage
-        stage1_model = self._kth_obj_model(k)
+    def _set_other_objectives_as_constraints(self, model, k, u, strict_inequality=True):
         for i in range(self._model.NumObj):
             if i != k:
                 other_objective = self._model.getObjective(i)
-                new_expression = gp.LinExpr()
-                for j in range(other_objective.size()):
-                    var = other_objective.getVar(j)
-                    coeff = other_objective.getCoeff(j)
-                    new_var = stage1_model.getVarByName(var.VarName)
-                    new_expression.add(new_var, coeff)
-                # Alter u[i] because Gurobi does not support strict < inequality
-                stage1_model.addLConstr(new_expression, GRB.LESS_EQUAL, u[i] - 0.5)
-        self._call_solver(stage1_model)
+                new_expression = self._new_expression_from_objective(model, other_objective)
+                try:
+                    rhs = u[i] - 0.5 if strict_inequality else u[i]
+                except Exception as e:
+                    breakpoint()
+                    1
+                model.addLConstr(new_expression, GRB.LESS_EQUAL, rhs)
 
-        # Second stage
-        stage2_model = gp.Model()
-        stage2_model.Params.OutputFlag = 0  # Suppress console output
-        self._copy_vars_to(self._model, stage2_model)
-        self._copy_constraints_to(self._model, stage2_model)
+    def _new_expression_from_objective(self, model, objective):
+        new_expression = gp.LinExpr()
+        for j in range(objective.size()):
+            var = objective.getVar(j)
+            coeff = objective.getCoeff(j)
+            new_var = model.getVarByName(var.VarName)
+            new_expression.add(new_var, coeff)
+        return new_expression
 
+    def _summed_expression_from_objectives(self, model, weights):
         coefficient_dict = {}
         for i in range(self._model.NumObj):
-            other_objective = self._model.getObjective(i)
-            new_expression = gp.LinExpr()
-            for j in range(other_objective.size()):
-                var = other_objective.getVar(j)
-                coeff = other_objective.getCoeff(j)
-                new_var = stage1_model.getVarByName(var.VarName)
-                new_expression.add(new_var, coeff)
+            objective = self._model.getObjective(i)
+            for j in range(objective.size()):
+                var = objective.getVar(j)
+                coeff = objective.getCoeff(j) * weights[i]
                 if var.VarName not in coefficient_dict:
                     coefficient_dict[var.VarName] = 0
                 coefficient_dict[var.VarName] = coefficient_dict[var.VarName] + coeff
-            if i == k:
-                rhs = round(stage1_model.ObjNVal)
-                constraint_sense = GRB.EQUAL
-            else:
-                rhs = self._eval_objective_given_model(stage1_model, self._model.getObjective(i))
-                constraint_sense = GRB.LESS_EQUAL
-            stage2_model.addLConstr(new_expression, constraint_sense, rhs)
-
-        # Set the objective for the stage 2 model
         summed_expression = gp.LinExpr()
         for varname in coefficient_dict:
-            new_var = stage2_model.getVarByName(varname)
+            new_var = model.getVarByName(varname)
             summed_expression.add(new_var, coefficient_dict[varname])
-        stage2_model.setObjective(summed_expression)
-        stage2_model.update()
-        return stage2_model
+        return summed_expression
+
+    def _upper_bounds_from_solved_model(self, model):
+        upper_bounds = []
+        for i in range(self._model.NumObj):
+            upper_bounds.append(self._eval_objective_given_model(model, self._model.getObjective(i)))
+        return tuple(upper_bounds)
 
     def _find_point(self, k, u):
         subproblem = self._construct_subproblem(k, u)
@@ -209,7 +219,15 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
             [self._eval_objective_given_model(subproblem, self._model.getObjective(i))
              for i in range(self._model.NumObj)]
         )
-        return new_point
+        decision_variables = self._var_values_by_name_dict(subproblem)
+        return (new_point, decision_variables)
+
+    def _remove_dominated(self, nds):
+        filtered = set()
+        for nd in nds:
+            if not any( (self.dominates(other, nd) for other in nds) ):
+                filtered.add(nd)
+        return filtered
 
     # Algorithm 2
     def find_non_dominated_objective_vectors(self):
@@ -228,8 +246,9 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
         while len(U) > 0:
             # Line 4
             k, u = self._next_k_u(U)
-            # Line 5 - but without specifying a starting solution for now
-            new_point = self._find_point(k, u)
+            # Line 5
+            new_point, decision_variables = self._find_point(k, u)
+            self._decision_variable_map[new_point] = decision_variables
             # Line 6
             V[k].add( (u, new_point[k]) )
 
@@ -263,4 +282,4 @@ class Tamby2020MOIPtimiser(MOIPtimiser):
                                     # Line 17
                                     U.remove(u_dash)
         # Line 18
-        return self._correct_sign_for_solutions(N)
+        return self._correct_sign_for_solutions(self._remove_dominated(N))
